using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Telerik.WinControls;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;
using System.Threading;

namespace VisualStyleBuilder.Design
{
	public partial class SplashForm : Form
	{
		private byte alpha = 30;
		private IntPtr handle;

		public SplashForm()
		{
			InitializeComponent();
		}

		protected override CreateParams CreateParams
		{
			get
			{
				CreateParams cp = base.CreateParams;
				cp.ExStyle |= NativeMethods.WS_EX_LAYERED;
				return cp;
			}
		}

		public override Image BackgroundImage
		{
			get
			{
				return base.BackgroundImage;
			}
			set
			{
				base.BackgroundImage = value;
				if (value != null)
					this.Size = this.BackgroundImage.Size;
			}
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			SetupLayeredWindow();
		}

		private void SetupLayeredWindow()
		{
			if (this.BackgroundImage as Bitmap == null || this.BackgroundImage.PixelFormat != PixelFormat.Format32bppArgb)
				return;

			this.handle = this.Handle;
			UpdateLayeredWindow(alpha);
			new Thread(new ThreadStart(animateFunc)).Start();
		}

		private void UpdateLayeredWindow(byte alpha)
		{
			Bitmap bitmap = (Bitmap)this.BackgroundImage;
			IntPtr hdcScreen = UnsafeNativeMethods.GetDC(new HandleRef(this, IntPtr.Zero));
			IntPtr hdcMemory = UnsafeNativeMethods.CreateCompatibleDC(new HandleRef(this, hdcScreen));
			IntPtr hBitmap = bitmap.GetHbitmap(Color.FromArgb(0));
			IntPtr hOldBitmap = SafeNativeMethods.SelectObject(new HandleRef(this, hdcMemory), new HandleRef(this, hBitmap));
			NativeMethods.SIZESTRUCT size = new NativeMethods.SIZESTRUCT(bitmap.Width, bitmap.Height);
			NativeMethods.POINTSTRUCT pointSource = new NativeMethods.POINTSTRUCT(0, 0);
			NativeMethods.POINTSTRUCT topPos = new NativeMethods.POINTSTRUCT(Left, Top);
			NativeMethods.BLENDFUNCTION blend = new NativeMethods.BLENDFUNCTION();
			
			blend.BlendOp = 0;
			blend.BlendFlags = 0;
			blend.SourceConstantAlpha = alpha;
			blend.AlphaFormat = 1;

			UnsafeNativeMethods.UpdateLayeredWindow(handle, hdcScreen, ref topPos, ref size, hdcMemory, ref pointSource, 0, ref blend, 0x00000002);
			UnsafeNativeMethods.ReleaseDC(new HandleRef(this, IntPtr.Zero), new HandleRef(this, hdcScreen));		
			if (hBitmap != IntPtr.Zero)
			{
				SafeNativeMethods.SelectObject(new HandleRef(this, hdcMemory), new HandleRef(this, hOldBitmap));
				SafeNativeMethods.DeleteObject(new HandleRef(this, hBitmap));
			}
			UnsafeNativeMethods.DeleteDC(new HandleRef(this, hdcMemory));
		}

		private void animateFunc()
		{
			while (alpha + 5 <= 255)
			{
				Thread.Sleep(5);
				alpha += 5;
				UpdateLayeredWindow(alpha);
			}
			if (alpha != 255)
				UpdateLayeredWindow(255);
		}
	}
}